/*
 * Copyright (c) Citrix Systems, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

#ifndef XEN_VGPU_H
#define XEN_VGPU_H

#include <xen/api/xen_common.h>
#include <xen/api/xen_gpu_group_decl.h>
#include <xen/api/xen_string_string_map.h>
#include <xen/api/xen_vgpu_decl.h>
#include <xen/api/xen_vm_decl.h>


/*
 * The VGPU class.
 * 
 * A virtual GPU (vGPU).
 */


/**
 * Free the given xen_vgpu.  The given handle must have been allocated
 * by this library.
 */
extern void
xen_vgpu_free(xen_vgpu vgpu);


typedef struct xen_vgpu_set
{
    size_t size;
    xen_vgpu *contents[];
} xen_vgpu_set;

/**
 * Allocate a xen_vgpu_set of the given size.
 */
extern xen_vgpu_set *
xen_vgpu_set_alloc(size_t size);

/**
 * Free the given xen_vgpu_set.  The given set must have been allocated
 * by this library.
 */
extern void
xen_vgpu_set_free(xen_vgpu_set *set);


typedef struct xen_vgpu_record
{
    xen_vgpu handle;
    char *uuid;
    struct xen_vm_record_opt *vm;
    struct xen_gpu_group_record_opt *gpu_group;
    char *device;
    bool currently_attached;
    xen_string_string_map *other_config;
} xen_vgpu_record;

/**
 * Allocate a xen_vgpu_record.
 */
extern xen_vgpu_record *
xen_vgpu_record_alloc(void);

/**
 * Free the given xen_vgpu_record, and all referenced values.  The
 * given record must have been allocated by this library.
 */
extern void
xen_vgpu_record_free(xen_vgpu_record *record);


typedef struct xen_vgpu_record_opt
{
    bool is_record;
    union
    {
        xen_vgpu handle;
        xen_vgpu_record *record;
    } u;
} xen_vgpu_record_opt;

/**
 * Allocate a xen_vgpu_record_opt.
 */
extern xen_vgpu_record_opt *
xen_vgpu_record_opt_alloc(void);

/**
 * Free the given xen_vgpu_record_opt, and all referenced values.  The
 * given record_opt must have been allocated by this library.
 */
extern void
xen_vgpu_record_opt_free(xen_vgpu_record_opt *record_opt);


typedef struct xen_vgpu_record_set
{
    size_t size;
    xen_vgpu_record *contents[];
} xen_vgpu_record_set;

/**
 * Allocate a xen_vgpu_record_set of the given size.
 */
extern xen_vgpu_record_set *
xen_vgpu_record_set_alloc(size_t size);

/**
 * Free the given xen_vgpu_record_set, and all referenced values.  The
 * given set must have been allocated by this library.
 */
extern void
xen_vgpu_record_set_free(xen_vgpu_record_set *set);



typedef struct xen_vgpu_record_opt_set
{
    size_t size;
    xen_vgpu_record_opt *contents[];
} xen_vgpu_record_opt_set;

/**
 * Allocate a xen_vgpu_record_opt_set of the given size.
 */
extern xen_vgpu_record_opt_set *
xen_vgpu_record_opt_set_alloc(size_t size);

/**
 * Free the given xen_vgpu_record_opt_set, and all referenced values. 
 * The given set must have been allocated by this library.
 */
extern void
xen_vgpu_record_opt_set_free(xen_vgpu_record_opt_set *set);


/**
 * Get a record containing the current state of the given VGPU.
 */
extern bool
xen_vgpu_get_record(xen_session *session, xen_vgpu_record **result, xen_vgpu vgpu);


/**
 * Get a reference to the VGPU instance with the specified UUID.
 */
extern bool
xen_vgpu_get_by_uuid(xen_session *session, xen_vgpu *result, char *uuid);


/**
 * Get the uuid field of the given VGPU.
 */
extern bool
xen_vgpu_get_uuid(xen_session *session, char **result, xen_vgpu vgpu);


/**
 * Get the VM field of the given VGPU.
 */
extern bool
xen_vgpu_get_vm(xen_session *session, xen_vm *result, xen_vgpu vgpu);


/**
 * Get the GPU_group field of the given VGPU.
 */
extern bool
xen_vgpu_get_gpu_group(xen_session *session, xen_gpu_group *result, xen_vgpu vgpu);


/**
 * Get the device field of the given VGPU.
 */
extern bool
xen_vgpu_get_device(xen_session *session, char **result, xen_vgpu vgpu);


/**
 * Get the currently_attached field of the given VGPU.
 */
extern bool
xen_vgpu_get_currently_attached(xen_session *session, bool *result, xen_vgpu vgpu);


/**
 * Get the other_config field of the given VGPU.
 */
extern bool
xen_vgpu_get_other_config(xen_session *session, xen_string_string_map **result, xen_vgpu vgpu);


/**
 * Set the other_config field of the given VGPU.
 */
extern bool
xen_vgpu_set_other_config(xen_session *session, xen_vgpu vgpu, xen_string_string_map *other_config);


/**
 * Add the given key-value pair to the other_config field of the given
 * VGPU.
 */
extern bool
xen_vgpu_add_to_other_config(xen_session *session, xen_vgpu vgpu, char *key, char *value);


/**
 * Remove the given key and its corresponding value from the
 * other_config field of the given VGPU.  If the key is not in that Map, then
 * do nothing.
 */
extern bool
xen_vgpu_remove_from_other_config(xen_session *session, xen_vgpu vgpu, char *key);


/**
 * .
 */
extern bool
xen_vgpu_create(xen_session *session, xen_vgpu *result, xen_vm vm, xen_gpu_group gpu_group, char *device, xen_string_string_map *other_config);


/**
 * .
 */
extern bool
xen_vgpu_destroy(xen_session *session, xen_vgpu self);


/**
 * Return a list of all the VGPUs known to the system.
 */
extern bool
xen_vgpu_get_all(xen_session *session, struct xen_vgpu_set **result);


#endif
